#include "MoleGraph.h"
#include "HX711.h"

//tare button
#define BUTTON_PIN 12

// Attache scales
HX711 scale(A2, A3);

unsigned counter =  0;
MoleGraph moleGraph;

float myForce = 0;             // filtered force (HX711)
float myForceRAW = 0;          // RAW force (HX711)
float myWeightRAW = 0;         // RAW force (HX711)
float myOffset = 0.0;          // Offset for Tare button 
int myForceTemp[3] = {0,0,0};  // force smoothing test array
int myForceI = 0;              // force smoothing counter
long previousTimer1 = 0;       //for HX711
long intervalTimer1 = 20;      //for HX711 - max 80 Hz, better 50 Hz >> 20 ms (25Hz is good >> 40 ms), best 8.33 Hz - 120 ms

// Variables will change:
int buttonState = 0;           // variable for reading the pushbutton status
int lastButtonState = 0;     // previous state of the button

void UpdateGraphChannels(void)
{
    // send forces    
    moleGraph.SetChannelValue(1, myForce);
    //moleGraph.SetChannelValue(2, myForceRAW);
    //moleGraph.SetChannelValue(3, myWeightRAW);
    // read and send the state of the pushbutton value:
    //moleGraph.SetChannelValue(4, buttonState-1);
}

void MeasurementStartedCallback(void)
{
  digitalWrite(13, HIGH);
}

void MeasurementStoppedCallback(void)
{
  digitalWrite(13, LOW);
}

void MeasurementPausedCallback(void)
{
  digitalWrite(13, LOW);
}

void MeasurementContinuedCallback(void)
{
  digitalWrite(13, HIGH);
}

void setup() 
{

  // initialize the pushbutton pin as an input:
  pinMode(BUTTON_PIN, INPUT);
  //digitalWrite(BUTTON_PIN, HIGH); 
  
  moleGraph.Setup();
  
  //Set callback functions which you will use  
  moleGraph.SetSendingCallback(&UpdateGraphChannels);
  moleGraph.SetMeasurementStartedCallback(&MeasurementStartedCallback);
  moleGraph.SetMeasurementStoppedCallback(&MeasurementStoppedCallback);
  moleGraph.SetMeasurementPausedCallback(&MeasurementPausedCallback);
  moleGraph.SetMeasurementContinuedCallback(&MeasurementContinuedCallback);
  
  scale.set_gain(128);         // set gain; see the README for details
  scale.set_scale(55.56);     // TFs val: -58.5, this value is obtained by calibrating the scale with known weights; see the README for details
  //scale.set_scale();
  scale.tare();                // reset the scale to 0
  
  myOffset = 0;
}

void loop() 
{
  //run my timers
  unsigned long currentTimer1 = millis();
  
  moleGraph.CheckInput();

  //Serial.println(buttonState);
  // test tare button
  tare_Button();

  //process actual force from sensor
  if(currentTimer1 - previousTimer1 > intervalTimer1) {      
   
    //get actual force      
    myForceTemp[myForceI] = calculateF();
    //save test RAW value
    //myForceRAW = (float)myForceTemp[myForceI]/100;
    if (myForceI == 0) myForceRAW = (float)myForceTemp[2]/100;
    if (myForceI == 1) myForceRAW = (float)myForceTemp[0]/100;
    if (myForceI == 2) myForceRAW = (float)myForceTemp[1]/100;
    
    myForceI++; 
            
    //median from 3
    myForce = (float)median_of_3(myForceTemp[0],myForceTemp[1],myForceTemp[2])/100;
    if (myForceI >= 3) {
      myForceI = 0;
    } 
                
    previousTimer1 = currentTimer1;
  }
  
}

void tare_Button() {
  // read the state of the pushbutton value:
  buttonState = digitalRead(BUTTON_PIN);

  // compare the buttonState to its previous state
  if (buttonState != lastButtonState) {
    // if the state has changed
    if (buttonState == HIGH) {
      // if the current state is HIGH then the button
      // wend from on to off:
      // tare scale
      //delay(50);
      scale.tare(); // reset the scale to 0
      myOffset = scale.get_units();
    } 
    //delay(50);
  }
  // save the current state as the last state
  lastButtonState = buttonState;    
}

int calculateF (){
  //calculate F
  myWeightRAW = scale.get_units() - myOffset;
  float F = 0.00981*myWeightRAW;
  F = round(F*100);
  int I = (int)F;
  return I;
}

int median_of_3(int a, int b, int c){
    int the_max = max(max(a, b), c);
    int the_min = min(min(a, b), c);
    // unnecessarily clever code
    int the_median = the_max ^ the_min ^ a ^ b ^ c;
    return(the_median);
}
